#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Stair Maker
#      By:	sdmitch
#   Usage:	Create simple straight, spiral, and u-shaped stair cases
#	 Note:	
#			
#    Date:	Aug 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
	sub = UI.menu("Plugins").add_submenu("Stair Maker")
	sub.add_item("Normal Stair") { SDM_SM.straight_run }
	sub.add_item("Spiral Stair") { SDM_SM.spiral_stair }
	sub.add_item("Ushape Stair") { SDM_SM.ushape_stair }
end
#
file_loaded(filename)
#
module SDM_SM

	def SDM_SM.straight_run

		ans=UI.inputbox(["Step Width:","Tread Depth:","Riser Height:","Num of Steps:"],[3.feet,1.feet,6.inch,9],"Stairs")
		if !ans then return end
		sr_width=ans[0].to_l;sr_tread=ans[1].to_l;sr_riser=ans[2].to_l;sr_steps=ans[3].to_i
		p0 = Geom::Point3d.new(0,0,0); pts=[];
		mod=Sketchup.active_model;ent=mod.entities
		mod.start_operation "Straight Run"
		steps=ent.add_group; sEnt=steps.entities; steps.name="Steps"
		rails=ent.add_group; rEnt=rails.entities; rails.name="Rails"
		for i in 1..sr_steps
			p1=p0.offset(Z_AXIS,sr_riser)
			p2=p1.offset(Y_AXIS,sr_tread)
			p3=p2.offset(X_AXIS,sr_width)
			p4=p1.offset(X_AXIS,sr_width)
			p5=p4.offset(Z_AXIS,3.feet)
			rEnt.add_line(p4,p5);pts.push p5;p0=p2
			face=sEnt.add_face(p1,p2,p3,p4)
			face.pushpull 2.inch if face
		end
		p5=p3.offset(Z_AXIS,3.feet)
		rEnt.add_line(p3,p5);pts.push p5
		rEnt.add_curve(pts); #add railing
		mod.commit_operation
	end

	def SDM_SM.spiral_stair
	
		prompts=["Max Width of Steps:","Min Width of Steps:","Riser Height:","Height of Stairs"]
		ans=UI.inputbox(prompts,[6.feet,3.feet,6.inch,9.feet],"Spiral Staircase")
		if !ans then return end
		mod=Sketchup.active_model; ent=mod.entities
		ss_width_max=ans[0].to_l; ss_width_min=ans[1].to_l
		ss_riser = ans[2].to_l; ss_height = ans[3].to_l;
		start_el = ss_riser; angle = 0; angle_increment=15
		p0=p3=Geom::Point3d.new(0,0,0);#origin
		xaxis=Geom::Vector3d.new(1,0,0); xaxis.length=ss_width_max+3.inch
		yaxis=Geom::Vector3d.new(0,1,0); yaxis.length=ss_width_min+3.inch
		zaxis=Geom::Vector3d.new(0,0,1);pts=[]
		mod.start_operation("Spiral Stair")
		steps=ent.add_group; sEnt=steps.entities; steps.name="Spiral Stairs"
		cir=sEnt.add_circle(p0,zaxis,3.inch,16)
		face=sEnt.add_face(cir); face.pushpull -(ss_height+ss_width_max)
		cosa = Math.cos(angle.degrees); sina = Math.sin(angle.degrees)
		vec = Geom::Vector3d.linear_combination(cosa,xaxis,sina,yaxis)
		start_el.step(ss_height,ss_riser) do |el|
			p1=p0.offset(zaxis,el)
			p2=p1.offset(vec); line=sEnt.add_line(p1,p2)
			angle += angle_increment; angle = 0 if angle >= 360
			cosa = Math.cos(angle.degrees); sina = Math.sin(angle.degrees)
			vec = Geom::Vector3d.linear_combination(cosa,xaxis,sina,yaxis)
			p3=p1.offset(vec); face=sEnt.add_face(p1,p2,p3); face.pushpull 2.inch
			p4=p2.offset(zaxis,3.feet); line=sEnt.add_line(p2,p4); pts.push p4; #add railing point
		end
		p4=p3.offset(zaxis,3.feet); sEnt.add_line(p3,p4); pts.push p4; #add railing point
		curve=sEnt.add_curve(pts); #add railing
		mod.commit_operation
	end
	
	def SDM_SM.ushape_stair
		prompts=["Step Width:","Tread Depth:","Riser Height:","Num of Steps:","Steps in Turn:"]
		ans=UI.inputbox(prompts,[3.feet,1.feet,6.inch,9,0],"U-Shaped Stairs")
		if !ans then return end
		sr_width=ans[0].to_l;sr_tread=ans[1].to_l;sr_riser=ans[2].to_l;sr_steps=ans[3].to_i; sr_tstep=ans[4].to_i
		p0 = Geom::Point3d.new(0,0,0);mod=Sketchup.active_model;ent=mod.entities;sel=mod.selection
		pts=[];mod.start_operation "ushape stair"
		steps=ent.add_group; sEnt=steps.entities
		for i in 1..sr_steps
			p1=p0.offset(Z_AXIS,sr_riser)
			p2=p1.offset(Y_AXIS,sr_tread)
			p3=p2.offset(X_AXIS,sr_width)
			p4=p1.offset(X_AXIS,sr_width)
			face=sEnt.add_face(p1,p2,p3,p4)
			face.pushpull 2.inch if face
			p5=p4.offset(Z_AXIS,3.feet)
			sEnt.add_line(p4,p5);pts.push p5;p0=p2
		end
		p5=p3.offset(Z_AXIS,3.feet)
		sEnt.add_line(p3,p5);pts.push p5
		curve=sEnt.add_curve(pts); #add top railing
		xgap=6.inch;zcpy=p1.z;zgap=0
		xgap=sr_tread*sr_tstep if sr_tstep > 0
		zgap=sr_riser*sr_tstep if sr_tstep > 0
		trn=Geom::Transformation.new([sr_width+xgap,0,zcpy+zgap])
		scpy=steps.copy; scpy.transform! trn
		trr=Geom::Transformation.rotation(scpy.bounds.center,Z_AXIS,180.degrees)
		scpy.transform! trr
		if sr_tstep == 0
			p1=p2
			p2=p1.offset(Y_AXIS,sr_width)
			p3=p2.offset(X_AXIS,sr_width*2.0+xgap)
			p4=p1.offset(X_AXIS,sr_width*2.0+xgap)
			face=sEnt.add_face(p1,p2,p3,p4)
			face.pushpull 2.inch if face
		else
			p0=p3;pts=[];p1=p2;p4=p3#add landing
			p2=p1.offset(Y_AXIS,sr_width)
			p3=p4.offset(Y_AXIS,sr_width)
			face=sEnt.add_face(p1,p2,p3,p4)
			face.pushpull -2.inch if face
			for i in 1..sr_tstep #add turn steps
				p1=p0.offset(Z_AXIS,sr_riser)
				p2=p1.offset(Y_AXIS,sr_width)
				p3=p2.offset(X_AXIS,sr_tread)
				p4=p1.offset(X_AXIS,sr_tread)
				face=sEnt.add_face(p1,p2,p3,p4)
				face.pushpull 2.inch if face
				p5=p1.offset(Z_AXIS,3.feet)
				sEnt.add_line(p1,p5);pts.push p5;p0=p4
			end
			p5=p4.offset(Z_AXIS,3.feet)
			sEnt.add_line(p4,p5);pts.push p5
			sEnt.add_curve(pts);#add top railing
			p1=p4.offset(X_AXIS,sr_width)
			p2=p3.offset(X_AXIS,sr_width)
			face=sEnt.add_face(p1,p2,p3,p4)
			face.pushpull -2.inch if face
		end
		sel.clear;dump=steps.explode;dump.each{|e| sel.add e if e.is_a?(Sketchup::Edge)||e.is_a?(Sketchup::Face)}
		dump=scpy.explode;dump.each{|e| sel.add e if e.is_a?(Sketchup::Edge)||e.is_a?(Sketchup::Face)}
		grp=ent.add_group sel; grp.name="U-Shaped Stairs"
		mod.commit_operation
	end
end